
class 'Markov'

  Markov.rootnotes = {}
  Markov.event = {}
  Markov.chain = {}
  Markov.chain2 = {}
  Markov.buf = {}
  
  Markov.ns = {}
  Markov.last_ns = ''
  
  Markov.PST_l = 3
  Markov.order = 1
  Markov.PST_minp = .15
  Markov.PST_r = 1
  Markov.IP_iter = 0
  Markov.IP_max_depth = 16
  
  Markov.continue = 1
  Markov.mul = false
  Markov.first_n_ix = nil
  
  Markov.params = ''
  Markov.typ = ''
  
  
local function getLenRow(len_tb)
  local line = 0  
  
  local last_ln = 0
  local last_k = 0
  
  for k,v in ipairs(len_tb) do
     
    if v > 0 then  
      line = line + v
      last_k = k
    end    
  end
  return last_k+1,line 
end


function Markov:paramsToStr()  
  local tb    
  
  if self.typ == 'IP' then      
    tb = {
       self.typ,
       self.order,
       self.IP_iter,
       self.IP_max_depth      
    }    
    return table.concat(tb,'_')
  else        
    local mul
    
    if self.mul then 
      mul = 't'
    else
      mul = 'f'
    end
    
    tb = {
       self.typ,
       self.order,
       mul,
       self.PST_l,
       self.PST_minp,
       self.PST_r
    }    
    return table.concat(tb,'_')
  end
end


function Markov:grabNotes(init)  
  return
end


function Markov:grabNotes2(init)  ---------del
  
  local ph  = MW.phr.no
  local col = MW.phr.col
  
  if self.event_type == 2 then
    
   -- initPoly() -- if init then initPoly() end
    
    for i=12,1,-1 do  
      if init then POLYptr[i] = Notes.new() end
            
      GrebSeq(i,false, MW.phr[ph].notes[ph])  
     end         
  elseif self.event_type == 3 then
    if init then CHptr = Chords.new() end
    
    GrebSeq(col, true, CHptr) 
  else 
    if init then Nptr = Notes.new() end

     GrebSeq(col, false, Nptr)  
  end
end


function getKeys(tb) ----------------------------------------------- Branches ---------------------------
  local keys = {}
  
  for k,v in pairs(tb) do 
    if type(v) == "table"  then
      table.insert(keys,k)
    end 
  end   
  return keys
end


function getRandBranch (tb, res_tb, last_branch, last_branch_key, first_key)   -- (tb,{},{},nil,nil)

  local keys = getKeys(tb)
  local rn
  
  local key, node = nil,nil
  
  if #keys > 0 then 
    rn = math.random(#keys)
    key = keys[rn]
    
    if first_key == nil then
      first_key = key
      
      last_branch = tb
    end            
    
    if #keys > 1 then
      last_branch_key = key
    end  
  elseif last_branch_key then 
    last_branch[last_branch_key] = nil
  elseif first_key then
    last_branch[first_key] = nil
  end 
   
  if tb[key] and type(tb[key]) == "table"  then  
    
    table.insert(res_tb,key)
        
    local branch_keys = getKeys(tb[key])  
    if #branch_keys > 1 then
      last_branch = tb[key]
    end    

    return getRandBranch (tb[key], res_tb, last_branch, last_branch_key, first_key)    
  end
 
  return res_tb, last_branch_key
end


function GetRandBranchSiblings (tb, res_tb, unexplored, first_key)

  local keys = getKeys(tb)
  local rn
  
  local key, node = nil,nil
  
  if #keys > 0 then  
    rn = math.random(#keys)
    key = keys[rn]               
    
    if first_key and #keys > 1 then
      for _,v in ipairs(keys) do       
        if v ~= key then           
                  
          local tb2 = {}
          tb2[v] = tb[v]
          table.insert(unexplored,tb2)
        end
      end  
    end    
    
    if first_key == nil then
      first_key = key
    end 
  elseif #keys == 0 then

    for i=#unexplored,1,-1 do
      GetRandBranchSiblings (unexplored[i], res_tb, {}, first_key) 
    end
    
    return res_tb, first_key
  end 
   
  if tb[key] and type(tb[key]) == "table"  then      
    table.insert(res_tb,key) 
    return GetRandBranchSiblings (tb[key], res_tb, unexplored, first_key)    
  end
 
  return res_tb, first_key
end


function flatenBranches (tb)
  local flat_tb = {} 

  while lenTb(tb) > 0 do
    local res_tb = getRandBranch (tb,{},{},nil,nil) 
    table.insert(flat_tb,res_tb) 
  end
  
  return flat_tb 
end


local function treeP (tb) ----------------------------------------- PST --------------------------
  if type(tb) == "table" then 
    for k,v in pairs(tb) do 
      if type(v) == "table" then           
        
        if v.p < self.PST_minp then
          print ( v.p )
        else 
          treeP(v)  
        end          
      end      
    end    
  end
end


local function pruneTree (tb,tb2,minp)
  if type(tb) == "table" then 
    for k,v in pairs(tb) do 
      if type(v) == "table" then           
        
        if v.p > minp then
          tb2[k] = v  
          pruneTree(v,tb2[k],minp)  
        end          
      end      
    end    
  end
end


local function PSTAddP (tb)
  if type(tb) == "table" then
    local len = 0       
    for k,v in pairs(tb) do 
     
      if type(v) == "table" then    
        len = len + v.count
      end   
    end      
    
    for k,v in pairs(tb) do 
      if type(v) == "table" then    
        v.p = v.count / len          
      end
      PSTAddP(v)
    end    
  end
end


local function buildPST (tb, keys)
  
  local key = next(keys)
   
  if key == nil then 
    return     
  elseif tb[keys[key]] == nil then
    local key2 = keys[key]
    tb[key2] = {}
    tb[key2].count = 1    

    keys[key] = nil    
    buildPST (tb[key2], keys)
  elseif tb[keys[key]] then
    local key2 = keys[key]

    tb[key2].count = tb[keys[key]].count + 1    
  
    keys[key] = nil
    buildPST (tb[key2] , keys)
  end  
end


local function rootNotes (tb)
  local ret_tb = {}
  if type(tb) == "table" then 
    for k,v in pairs(tb) do 
      if (type(v) == "table" and k ~= 'p' and k ~= 'count') then    
        table.insert(ret_tb, k)                
      end
    end    
  end
  return ret_tb
end


function Markov:buildNotesTreePST() 
   
  local nn = self.PST_l
  local order = self.order - 1
  
  if self.mul and self.PST_l > 1 then
    nn = 1
  end

  for _,one_NS in ipairs(Stats.note_strings) do  
    for len=nn,self.PST_l do
      for i=1,#one_NS do                      
        local motif = {unpack(one_NS,i,i+len+order)}     
 
        if order > 0 then
          local head = reduce({unpack(motif, 1, self.order)}, function (ns1,ns2) return ns1 .. '|' .. ns2 end) 
          motif = cons({unpack(motif, self.order+1, #motif)}, head)  
        end
        
        if #motif == self.PST_l+1 then buildPST (self.chain, motif) end   
      end   
    end
  end  
  
  PSTAddP (self.chain) 
   
  local markov_pruned = {}  
  pruneTree (self.chain, markov_pruned, self.PST_minp) 

  self.chain = markov_pruned
end


function IterIP (tb,order,IP_iter) 
  local i = 0
  local j = 0
  local new_iter = false
  local node_counter = 0 
  
  local o = {}
  
  function o.inc()
    node_counter = node_counter + 1
  end
   
  function o.reset()
    node_counter = 0
  end
  
  function o.node_count()
    return node_counter
  end
  
  function o.iteration_no ()
    return j
  end   
    
  function o.next(depth) 
    if i == #tb and (j < IP_iter or IP_iter == 0) then
      i=0
      j=j+1
      new_iter = true
    else
      new_iter = false
    end
    
    i = i + 1
    
    local en = i
    local gram
     
    if depth == 0 then
      en = i + order - 1 
    end 
    
    if depth == 0 and i == #tb and order > 1 then
      local extend = #tb % order
      local tb1 = { unpack(tb,i,en) }
      local tb2 = { unpack(tb,1,order-1) }
      
      gram = table.concat(tableConcat( tb1,tb2), '|')
      i = order-1
      new_iter = true
    else 
      gram = table.concat({ unpack(tb,i,en) }, '|')
    end

    if gram then
      return gram,new_iter      
    else
      return nil,nil  
    end 
  end
  
  
  function o.keep_going()
    if i <= #tb and (j < IP_iter or IP_iter == 0) then
      return true 
    else
      return false  
    end     
  end
  
  return o
end


function nodeIP (tb,fun,depth,max_depth)  
  local val,new_iter = fun.next(depth)
  
  if new_iter then  
    sprint('Building notes dictionary, IP iteration ' .. fun.iteration_no() .. '  added ' .. fun.node_count() .. ' nodes')
    if fun.node_count() == 0 then
      return false
    end   
    fun.reset() 
  end

  if val and (depth <= max_depth or max_depth == 0) then
    if tb[val] == nil then

      tb[val] = {}     
      fun.inc()
      return true
    else            
      depth = depth + 1
      return nodeIP (tb[val],fun,depth,max_depth)    
    end
  else 
    return false
  end
end


local function nodesToNotesIP (tb,ns_tb)  
  if type(tb) == "table" then 
    
    local keys = {}
    
    for k,v in pairs(tb) do 
      if type(v) == "table"  then    
        table.insert(keys, k)                
      end      
    end  
    
    if #keys > 0 then
      local ix = math.random(#keys)    
      table.insert(ns_tb, keys[ix])  
      
      nodesToNotesIP(tb[keys[ix]],ns_tb)  
    end
  end
end


local function nodesToNotesIP2 (tb,res)  
  if type(tb) == "table" then 
    
    local keys = {}
    
    for k,v in pairs(tb) do 
      if type(v) == "table"  then    
        table.insert(keys, k)                
      end      
    end  
    
    if #keys > 0 then
      local ix = math.random(#keys)    
      table.insert(res, keys[ix])  
      
      return nodesToNotesIP2(tb[keys[ix]],res)  
    end
  end
  return res
end


function Markov:buildNotesTreeIP()   
  self.chain = {}
  
  for _,one_NS in ipairs (self.ns) do  
    local fun = IterIP (one_NS, self.order, self.IP_iter) 
  
    while fun.keep_going() do
      local new_nodes = nodeIP (self.chain ,fun, 0, self.IP_max_depth) 
      
      if not new_nodes then 
        sprint('No new nodes found in new IP iteration, exiting at ' .. fun.iteration_no())
        break 
      end
    end 
  end     
end


local function remEmptyIP (tb)
  if type(tb) == "table" then 
    for k,v in pairs(tb) do 
      if next(v) == nil  then    
        tb[k] = nil                
      end
    end    
  end
end


local function seachTbForKeys (tb,key_tb)
  
  if type(tb) == "table"  then
  
    for k,v in pairs(tb) do 
      if type(v) == "table" then  
               
        local key = head(key_tb) 
        
        if v[key] and #key_tb == 1 then         
          return v[key]   
        elseif v[key] then  
          return seachTbForKeys(v,tail(key_tb))  
        else
          return seachTbForKeys(v,key_tb)
        end                 
      end      
    end    
  end 
end


local function nstrToChrdNotes(ns) --------------------- N to STR and back -----------------------------
  local tb = splitDl (ns, ',')  
    
  local chrd = splitDl (tb[1], '_')   
  local len = tonumber(tb[2])  

   return map(chrd,function(v) return tonumber(v) end), len  
end


local function keyToNL(ns) 
  local tb = splitPunRetNum(ns)  
  return tb[1],tb[2]
end


local function nstrToNote (ns) 
  local tmp_note, len = keyToNL(ns)
  local note, oct = oct0 (tmp_note) 
  return note, oct, len
end


local function startNtb (rootnotes, event_type) 

  local tb = {'Random'}
  local N_tb = {}
  
  for _,v in ipairs(rootnotes) do  
    
    local str = ''
      
      if event_type == 2 then     
        N_tb = splitDl (v, ';') 
      else
        N_tb = splitDl (v, '|')
      end
      
      if #N_tb == 1 then          
      
        if event_type == 3 then 
          local chrd = nstrToChrdNotes (N_tb[1])

          for _,chrd_note in ipairs(chrd) do
            str = str .. num2note(chrd_note) .. '  '
          end
        else
          local note, oct = nstrToNote (N_tb[1])
          str = num2note(note + oct*12)
        end
                    
      else -- higher order, more than one N  
      
        for i,vv in ipairs(N_tb) do
          if MW.phr.editor_chords then       
            local chrd = nstrToChrdNotes (vv)

            if i==#N_tb then 
              str = str .. table.concat(chrd,' ')   
            else
              str = str .. table.concat(chrd,' ') .. ',  '
            end
          else
            local note, oct = nstrToNote (vv)  
            
            if i==#N_tb then  
              str = str .. num2note(note + oct*12)
            else
              str = str .. num2note(note + oct*12) .. ',  '            
            end
          end
        end
      end
 
    table.insert(tb,str) 
  end
    
  UI.markov_first_N.items = tb   
end

    
function Markov:addToBuf (ns,ns_tb)
  if #ns_tb > 0 then 
    
    local len = #self.buf
    
    if len > 10 then
      self.buf = {unpack(self.buf,len - 9, len)}  
    end
    
    each(splitDl (ns, '|'), function (ns) table.insert(self.buf,ns) end)
    each(ns_tb, function (ns) table.insert(self.buf,ns) end)
  end
end


function Markov:getLastBufN ()
  if #self.buf >= self.order then    
    return reduce( {unpack(self.buf, #self.buf-self.order+1, #self.buf)}, function(ns1,ns2) return ns1 .. '|' .. ns2 end)
  end
end
        

function Markov:branchToNotes (ns)
  
  local ns2
  local tb2 = {}
  
  if self.order > 1 then   
    ns2 = splitDl (ns, '|') 
    
    if self.typ == 'IP' then ns = ns2[1] end 
  else
    ns2 = ns 
  end
   
  if self.chain[ns] then    
    self.last_ns = ns        
    return nodesToNotesIP2 (self.chain[ns],{})  
  end


  local tb = copy(self.chain)

  while lenTb(tb) > 0 do 
    local branch = getRandBranch (tb,{},{},nil,nil) 
      
    if #branch > 0 then  
      tb2 = tablesCompRet (ns2,branch)
        
      if #tb2>0 then 
        return tb2 
      else
        tb2 = {}  
      end   
    end   
  end   
        
                                 
  if #tb2 == 0 then  
    ns = randomKey(self.chain)
    
    local i = 10 
    while ns == self.last_ns do 
      if i<0 then break end
      ns = randomKey(self.chain)
      i=i-1
    end     
    self.last_ns = ns        
    
    return nodesToNotesIP2 (self.chain[ns],{})   
  end
end


function Markov:PST_IP_event(ns,line,i)
  
  local o  = self.dst
  local ph = MW.phr.no 
  local note, oct, len, ch_notes  
  
  o.off[i]  = 0
  o.vel[i]  = 128 
  
  if self.event_type == 2 then      
     local poly_tb = splitDl (ns, ';')              
            
     local fun = function(a,b) 
       local _, _, len_a = nstrToNote (a)  
       local _, _, len_b = nstrToNote (b)        

       return len_a<len_b
     end
             
    table.sort(poly_tb, fun) 
     
    for n_col,k in ipairs(poly_tb) do  ---------------------------------------------
      note, oct, len = nstrToNote(k)  
              
      if n_col == 1 then 
        o[n_col].oct [i] = oct  
        o[n_col].note[i] = noteInScale(note)         
                
        if i==1 then Nptr.len[i] = 1 end

        o[n_col].len[i+1] = len 
        o[n_col].off[i]   = len    
      else            
          local li, last_ln = getLenRow(o[n_col].len)  
          local new_ln = line - last_ln  
              
          o[n_col].note[li] = noteInScale(note)              
             
          if new_ln + len < self.pat_len then
            o[n_col].off[li] = new_ln + len 
          end
            
          o[n_col].len[li] = new_ln   
          o[n_col].oct[li] = oct 
      end            
    end
     
  elseif self.event_type == 3 then  
    ch_notes, len = nstrToChrdNotes(ns)     

    if i == 1 then len = 1 end   
        
    o.len[i]   = len
    o.notes[i] = ch_notes    
         
  else --- mono
    note, oct, len = nstrToNote(ns)    
      
    if i == 1 then len = 1 end      

    o.note[i] = note
    o.oct[i] = oct
      
    o.len[i] = len        
  end
  
  return line + len
end


function Markov:fill(note_col)

  if self.event_type == 3 then         
  
    fillPtr(note_col, CHptr, 3) 
    
  elseif self.event_type == 2 then  
  
    fillPtr(note_col, POLYptr[note_col], 2)     
  else    
    fillPtr(note_col, Nptr, 1)
  end
end


function Markov:PST_IP_Run(...)   --------------------------- PST / IP ---------------------------------------------
  
  local ph = MW.phr.no
  
  local msg = '' 
  local key              
                               
  if self.typ == 'PST' then    
    msg = 'PST - Did not generate any notes. Try decreasing PST L or PST R or increasing PST P '
  elseif self.typ == 'IP' then
    msg = 'IP - Not enough notes to pharse'
  end
  
  if #self.rootnotes < 1 then 
    sprint (msg) 
    return 
  else      
     -- self:grabNotes(false)  
      
    if self.event_type == 2 then
      initPoly()
      emptyTr()
    end
    startNtb (self.rootnotes, self.event_type)
  end

    
  local i = 1
  local line = 1 

  if self.first_n_ix then
    key = self.rootnotes[self.first_n_ix]   
    sprint('Start note ' .. key)  
    
    for _,ns in ipairs(splitDl(key,'|')) do
      line = self:PST_IP_event(ns,line,i)
      i=i+1
    end
  else
    key = self.rootnotes[math.random(#self.rootnotes)]  
  end
           
  local root_branch = self.chain[key]      
  while line <= self.pat_len do   
  
    local ns_tb
        
    if arg[1] then 
      ns_tb = arg[1]
      arg[1] = nil
    else
      ns_tb = self:branchToNotes (key) 
    end

    Markov:addToBuf (key,ns_tb) 
        
    for k,ns in pairs(ns_tb) do  
     
      line = self:PST_IP_event(ns,line,i)
      
      if line > self.pat_len then break end
   
      key = Markov:getLastBufN () 
              
      if self.chain[key] then
        if key == ns_tb[1] then
          key = randomKey(self.chain)
        end
      end      

      i=i+1    
    end          
  end  
  
  if self.event_type == 2 then
    for i=1,12 do
      assignNoteTbs(i)
      self:fill(i)      
    end
  else
    self:fill(MW.phr.col)
  end
  
  --if UI.block_selection then self:grabNotes(true) end 
  
  updateSlots() 
end
 

function Markov:PST_IP_Build()

  self.chain, self.rootnotes, self.buf = {},{},{}
  
  if self.typ == 'PST' then  
    self:buildNotesTreePST() 
  elseif self.typ == 'IP' then
    self:buildNotesTreeIP()
  end  
    
  remEmptyIP (self.chain)  
  self.rootnotes = rootNotes(self.chain) 
  
  self.chain2 = copy(self.chain)   
end


function Markov:buildMPLchain(L) --------------------------------- MLP

  local order = self.order - 1 

  for _,one_NS in ipairs(self.ns) do  
    for i=1,#one_NS do      

      local N_r, motif
    
      local tb = {unpack(one_NS, i, i+order)}
      local tb2  = {unpack(one_NS, i+self.order, i+order+L)}

      if #tb == self.order and #tb2 == L then 
        N_r  = reduce(tb, function (ns1,ns2) return ns1 .. '|' .. ns2 end) 
        motif = reduce(tb2, function (ns1,ns2) return ns1 .. '|' .. ns2 end)
    
        if self.chain [N_r] == nil then
          self.chain [N_r] =  {}        
        
          self.rootnotes[N_r] =  {}
          self.rootnotes [N_r].count =  1
        elseif self.rootnotes [N_r] then
          self.rootnotes [N_r].count =  self.rootnotes [N_r].count + 1
        end  
      
        if self.chain [N_r] [motif] == nil then
          self.chain [N_r] [motif] = {}
          self.chain [N_r][motif] =  {count = 1}  
        else
          local n = self.chain [N_r] [motif].count
          self.chain [N_r] [motif].count = n + 1
        end
      end
    end  
  end  
end


function Markov:branchToNotesMLP(ns)  
   
  local tb2 = {}  
   
  if self.chain[ns] then 
    self.last_ns = ns 
    
    return splitDl (randomKey(self.chain[ns]), '|')  
  end   
   
  local markov_tb = copy(self.chain)   
  local ns2
  
  while lenTb(markov_tb) > 0 do 

    ns2 = randomKey(markov_tb) 
    
    local tmp = markov_tb[ns2]   
    local branch = splitDl (randomKey(tmp), '|')  
    markov_tb[ns2] = nil    
      
    if #branch > 0 then  
      if self.order > 1 then    
        ns2 = splitDl(ns2, '|')     
      end

      tb2 = tablesCompRet (ns2 ,branch)  
       
      if #tb2>0 then return tb2 end       
    end   
  end   
        
                                 
  if #tb2 == 0 then 
    ns = randomKey(self.chain)
    
    local i = 10 
    while ns == self.last_ns do  
      if i<0 then break end
      ns = randomKey(self.chain)
      i=i-1
    end
    self.last_ns = ns 
    
    return splitDl (randomKey(self.chain[ns]), '|')    
  end
end


function Markov:MPL_event (v,line,i)   

  local o = self.dst
   
  local note, oct, len, ch_notes
  
  o.off[i]  = 0
  o.vel[i]  = 128  
      
  if self.event_type == 2 then  
    
    local poly_tb = splitDl (v, ';')               
        
    local fun = function(a,b) 
      local _, _, len_a = nstrToNote (a)  
      local _, _, len_b = nstrToNote (b)        
      return len_a<len_b
    end
         
    table.sort(poly_tb, fun)    

    for n_col,k in ipairs(poly_tb) do  
      note, oct, len = nstrToNote (k)  
          
      if n_col == 1 then  
        o[n_col].oct[i] = oct       
        o[n_col].note[i] = noteInScale(note)  
                   
        if i==1 then Nptr.len[i] = 1 end
       
        o[n_col].len[i+1] = len  
        o[n_col].off[i]   = len                  
      else 
        local li, last_ln = getLenRow(o[n_col].len)  
           
        local new_ln = line - last_ln   
        o[n_col].note[li] = noteInScale(note)
           
        if new_ln + len < self.pat_len then             
          o[n_col].off[li] = new_ln + len 
        end
           
        o[n_col].len[li] = new_ln 
        o[n_col].oct[li] = oct               
      end              
    end

  elseif self.event_type == 3 then  
  
    ch_notes, len = nstrToChrdNotes(v)  
    
    if i == 1 then len = 1 end 
    
    o.len[i]   = len
    o.notes[i] = ch_notes   
     
  elseif self.event_type == 1 then 
  
    note, oct, len = nstrToNote(v)                                 
       
    if i == 1 then len = 1 end 
    
    o.oct[i]  = oct
    o.len[i]  = len      
    o.note[i] = noteInScale(note)           
  end
  
  return line + len  
end


function Markov:MLP_Run(...) 

  local n_col = MW.phr.col 

  local notes = {}  
  for k,v in pairs(self.chain) do        
    table.insert(notes,k)  
  end
  
  if #notes < 1  then 
    sprint('MPL not enough notes')  
    return 
  else
   --   self:grabNotes(false)
    
    if self.event_type == 2 then
      initPoly()
      emptyTr()
    end    
    startNtb (notes, self.event_type) 
  end
  
  local i = 1
  local line = 1 


  local key = notes[math.random(#notes)] 

  if self.first_n_ix then
    key = notes[self.first_n_ix]      
          
    for _,ns in ipairs(splitDl(key,'|')) do
      line = self:MPL_event(ns,line,i)    
      i=i+1
    end   
    
    sprint('Start note ' .. key)
  end


  while  line <= self.pat_len do

    if i > #Nptr.note then
      growSeqTbs(Nptr.len, Nptr.note, Nptr.off, Nptr.oct, Nptr.vel)
      growChrdTbs() 
    end 

    local tb = {}      
    local motif = self:branchToNotesMLP(key)  
    
    if arg[1] then 
      motif = arg[1]
      arg[1] = nil   
    end
      
    self:addToBuf (key,motif)  

    for k,v in ipairs(motif) do      
      line = self:MPL_event(v,line,i)  
      i=i+1
    end
        
    key = self:getLastBufN () 
      
    if self.chain[key] then
      if key == motif[1] then
        key = randomKey(self.chain)
      end
    end

    if line > self.pat_len then break end        
  end

  if self.event_type == 2 then 
    for i=1,12 do
      assignNoteTbs(i)                
      self:fill(i)      
    end  
  else
    self:fill(n_col)
  end
  
 -- if UI.block_selection then self:grabNotes(true) end
  
  updateSlots()   
end


function Markov:MLP_Build() 

  self.chain, self.rootnotes, self.buf = {},{},{} 
  
  if self.mul and self.PST_l > 1 then  
    for i=1,self.PST_l do
      self:buildMPLchain(i)  
    end
  else
    self:buildMPLchain(self.PST_l)    
  end

  local total = 0
  for v in pairs(self.rootnotes) do
    total = total + self.rootnotes[v].count 
  end
  
  for v in pairs(self.rootnotes) do
    self.rootnotes[v].p = self.rootnotes[v].count / total
  end
  
  for v in pairs(self.chain) do
    for vv in pairs(self.chain[v]) do
      self.chain [v] [vv].p = self.chain [v] [vv].count / self.rootnotes[v].count
    end
  end
    
  for v in pairs(self.rootnotes) do
    if self.rootnotes[v].p < self.PST_minp then
      self.rootnotes[v] = nil
    end
  end  

  for v in pairs(self.chain) do
    for vv in pairs(self.chain[v]) do
      if self.chain [v][vv].p < self.PST_minp or self.chain [v][vv].count < self.PST_r then
        self.chain [v][vv] = nil
      end
    end
  end  

  for v in pairs(self.chain) do
    if not next(self.chain[v]) then
      self.chain[v] = nil
    end
  end     
  
  self.chain2 = copy(self.chain)  
end
 
 
function Markov:__init(pat_len, ns, etype, dst) 
 
  self.ns        = ns       
  self.dst       = dst   
  self.pat_len   = pat_len  
  self.continue  = 1
  self.event_type = etype
  
  self.first_n_ix = nil
end


function Markov:run()
  local params = self:paramsToStr()  

  if string.find(self.params, params) == nil then  
    if self.typ == 'PST' or self.typ == 'IP' then
      self:PST_IP_Build()
      self:PST_IP_Run()
    else
      self:MLP_Build()
      self:MLP_Run()
    end
    
    self.params = params
    
  else  
  
    if self.typ == 'PST' or self.typ == 'IP' then
      self:PST_IP_Run()
    else
      self:MLP_Run()
    end
  end
end


function Markov:checkParams()   
  local params = Markov:paramsToStr() 

  if string.find(self.params, params) == nil  then 
    if self.typ == 'PST' or self.typ == 'IP' then
      self:PST_IP_Build()
    else
      self:MLP_Build()  
    end
    
    self.params = params
  end
end


function continueMarkov (n_tb) 

  if Scan.Markov_process then  
    Scan.Markov_process:checkParams()  
    
    if #Scan.Markov_process.ns == 0 or lenTb(Scan.Markov_process.chain2) == 0 then 
      sprint('Empty') 
      return
      --Scan.Markov_process:checkParams()  
    end 
  else 
    return   
  end
  
  local block_notes = copy(n_tb)

 -- local oct 
  for k,v in ipairs(n_tb) do  
    n_tb[k] = copy(map(v, function(a) return a%12 end))
    table.sort(n_tb[k])
    table.sort(block_notes[k])        

  end 

  local src_ns = table.concat(map(n_tb,function(v) return table.concat(v,'_') end), '|')  
  local scr_head 

  while Scan.Markov_process.chain2 do
    local nodes ,first_node  = GetRandBranchSiblings (Scan.Markov_process.chain2,{},{},nil)   
    local node_norm_tb, node_ns_tb = {},{}
    
    if  Scan.Markov_process.typ == 'MLP' then
      for k,v in ipairs(nodes) do  
        each(splitDl(v,'|'), function(v) table.insert(node_ns_tb, v) end)
      end        
      nodes = node_ns_tb    
    end  
    
      for k,v in ipairs(nodes) do  
        local node_ns
    
        if  Scan.Markov_process.event_type == 3 then      
          local tb = splitDl(v,',')
          node_ns = splitDl(tb[1],'_') 
        elseif Scan.Markov_process.event_type == 2 then  
          local tb = splitDl(v,';') 
          node_ns = map(tb, function(nl) local tb = splitDl(nl,'_') return tb[1] end)           
        else        
          local tb = splitDl(v,'|')
          node_ns = map(tb, function(nl) local tb = splitDl(nl,'_') return tb[1] end)  
        end 
      
        node_ns = copy(map(node_ns,function(v) return tonumber(v)%12 end))    
        table.sort(node_ns)
        node_norm_tb[k] = table.concat(node_ns, '_')   
      end 

    if Scan.Markov_process.typ == 'PST' or Scan.Markov_process.typ == 'IP' then    
       
      for i=1,#node_norm_tb do
        local ns_head = table.concat({unpack(node_norm_tb, i, i+Scan.Markov_process.order-1)}, '|') 
        local ns_res = {unpack(nodes, i, #nodes)}     

        if src_ns == ns_head and #ns_res > #ns_head then 
          Scan.Markov_process:PST_IP_Run(ns_res) 
          return
        end           
      end                      
    elseif Scan.Markov_process.typ == 'MLP' then
    
      for i=1,#node_ns_tb do
        local ns_head = table.concat({unpack(node_norm_tb, i, i+Scan.Markov_process.order-1)}, '|')   
        local ns_tail = {unpack(node_ns_tb, i+Scan.Markov_process.order, #node_ns_tb)}      

        if src_ns == ns_head and #ns_tail > 0 then        
          ns_head = {unpack(node_ns_tb, i, i+Scan.Markov_process.order-1)}
          Scan.Markov_process:MLP_Run( tableConcat(ns_head,ns_tail) )      
          return
        end 
      end       
    end

    if first_node then
      Scan.Markov_process.chain2[first_node] = nil
    else
      break
    end    
  end  
  
  sprint('Empty')
end


function grabBlockNLcontinueExact() 

  if Scan.Markov_process then  
    Scan.Markov_process:checkParams()  
    
    if #Scan.Markov_process.ns == 0 or lenTb(Scan.Markov_process.chain2) == 0 then 
      sprint('Empty')
      return 
    end 
  else 
    return   
  end
  
  local pat = curPattern ()  
  local sel = renoise.song().selection_in_pattern 

  local n_tb, l_tb, ns_tb = {},{},{}  
  
  for tr = sel.start_track, sel.end_track do
    for ln = sel.start_line, sel.end_line do
      local n_col_tb = {}
      
      for n_col = sel.start_column, sel.end_column do      
        local note = pat:track(tr):line(ln):note_column(n_col).note_value
         
        if (note < 120) then 
          table.insert(n_col_tb,note)            
        end
      end
      
      if #n_col_tb > 0 then
        table.insert(n_tb, n_col_tb)   
        l_tb[#n_tb] = ln
      end
    end    
  end
  
  if #n_tb == 0 then return end  

  for i,n_row in ipairs(n_tb) do
      local len
      
      if i<#n_tb then
        len = l_tb[i+1] - l_tb[i] 
      else
        len = sel.end_line - sel.start_line + 1
      end
      
      if Scan.Markov_process.event_type == 3 then
        ns_tb[i] = table.concat(n_row,'_') 
        ns_tb[i] = ns_tb[i] .. ',' .. len
      elseif Scan.Markov_process.event_type == 2 then
        local tb = map(n_row, function(v) return v .. '_' .. len end)
        ns_tb[i] = table.concat(tb,';')
      else
        ns_tb[i] = n_row[1] .. '_' .. len    
      end

  end

  local ns = table.concat({unpack(ns_tb,1,Scan.Markov_process.order)},'|')

  while Scan.Markov_process.chain2 do
    local nodes ,first_node  = GetRandBranchSiblings (Scan.Markov_process.chain2,{},{},nil) 
    local node_ns_tb = {}

    if  Scan.Markov_process.typ == 'MLP' then
      for k,v in ipairs(nodes) do  
        each(splitDl(v,'|'), function(v) table.insert(node_ns_tb, v) end)
      end 
      nodes = node_ns_tb       
    end  

    for i=1,#nodes do
      local ns_head = table.concat({unpack(nodes, i, i+Scan.Markov_process.order-1)}, '|')        
      local chain = {unpack(nodes, i, #nodes)} 

      if ns == ns_head then          
        if Scan.Markov_process.typ == 'PST' or Scan.Markov_process.typ == 'IP' then    
          Scan.Markov_process:PST_IP_Run(chain) 
          return
        elseif  Scan.Markov_process.typ == 'MLP' then   
          Scan.Markov_process:MLP_Run(chain)        
          return
        end
      end
    end

    if first_node then
      Scan.Markov_process.chain2[first_node] = nil
    else
      break
    end    
  end  
    
  sprint('Finished, empty')
end


function MarkovProcess()  

  local pat_len = renoise.song().selected_pattern.number_of_lines
  
  if #Stats.note_strings == 0 then return end
  
  if Scan.Markov_process == nil then 
    
    if MW.phr.editor_chords then
    
      initChords()
      Scan.Markov_process = Markov(pat_len, Stats.note_strings, 3, CHptr)
      
    elseif MW.phr.editor_poly then
    
      initPoly()
      Scan.Markov_process = Markov(pat_len, Stats.note_strings, 2, MW.phr[MW.phr.no].notes)
      
    else              
    
      initNotes()
      Scan.Markov_process = Markov(pat_len, Stats.note_strings, 1, Nptr)
      
    end
    
    Scan.Markov_process:run()
  else 
    Scan.Markov_process:run()
  end
  
  if #Stats.note_strings == 1 then
    sprint('Generate notes from ' ..  #Stats.note_strings .. ' phrase')
  else
    sprint('Generate notes from ' ..  #Stats.note_strings .. ' phrases')
  end    
end

