
function ScannerH2 (o, Line_tb, typ) 
  local line  = 0
  local notes = {}  
  
  for i,len in ipairs(o.len) do  
    local dur = 0
    local str
    
    line = line + len      
 
    if len > 0 then
      if (o.off[i] > 0) then 
        dur = o.off[i]
      elseif i < #o.len and o.len[i+1] > 0 then  
        dur = o.len[i+1]
      else  
        dur = o.pat_len - line + 1
      end     
    end

    if dur > 0 then
           
      if typ == 'mono' then  
        local note = o.note[i] % 12
        
        table.insert(Stats.unique_notes, noteInScale(note%12))      
        table.insert(notes,{line = line, note =  note, dur = dur})
        Stats.pitch_duration_dst[note+1] = Stats.pitch_duration_dst[note+1] + dur 
                
      elseif typ == 'chord' then
        for k,v in ipairs(o.notes[i]) do  
          local note = v % 12
          table.insert(Stats.unique_notes, noteInScale(note))
          table.insert(notes,{line = line, note =  note, dur = dur})  
          Stats.pitch_duration_dst[note+1] = Stats.pitch_duration_dst[note+1] + dur
        end        
      end
    end        
  end  
  
  return notes, line
end


function ScannerH() 
  local line  = 0  
  local notes = {}
  
  Stats.unique_notes       = {}
  Stats.pitch_duration_dst = {0,0,0,0,0,0,0,0,0,0,0,0}
  
  if MW.phr.event_type == 1 then
  
    notes =  ScannerH2 (Nptr, 0, 'mono')      
  
  elseif MW.phr.event_type == 2 then
   
    for i=1,12 do    
      local tb
      local ptr = MW.phr[MW.phr.no].notes[i]
            
      tb, line =  ScannerH2 (ptr, line, 'mono')     
      notes = tableConcat(notes, tb)
    end  
  end
  
  table.sort(notes, function(a,b) return a.line < b.line end) 
  return notes  
end


function ScannerChordH() 
  Stats.unique_notes       = {}
  Stats.pitch_duration_dst = {0,0,0,0,0,0,0,0,0,0,0,0}
 
  local notes = ScannerH2(CHptr, 0, 'chord')    
  table.sort(notes, function(a,b) return a.line < b.line end) 
  return notes  
end


function phraseScannerMonoChrd (o, Line_tb)  

  local Markov_NS = {}
  local Markov_LN = {}
  
  local line = 0
  
  local min_len 
  if MW.phr.editor_chords then
    min_len = math.min( unpack({#o.len, #o.notes}) )
  else
    min_len = math.min( unpack({#o.len, #o.note}) )
  end
  
  for i=1,min_len do  
    local len = o.len[i]
    local dur = 0
    local str
    
    line = line + len      
 
    if len > 0 then  
      if (o.off[i] > 0) then 
        dur = o.off[i]
      elseif i < #o.len and o.len[i+1] > 0 then  
        dur = o.len[i+1]
      else  
        dur = o.pat_len - line + 1
      end     
    end

    if dur > 0 then
    
      if Stats.note_strings_length ~= '' and Scan.scale_key_on == false then
        local len_tb = splitDl (Stats.note_strings_length, ',')
        local _, dur2 = nearestValue(len_tb, dur) 
        dur = dur2       
      end     
    
      if Rand.new_add_lock ~= 3 then 
        if Stats.note_len_distribution[dur] then
          Stats.note_len_distribution[dur] = Stats.note_len_distribution[dur] + 1
        else
          Stats.note_len_distribution[dur] = {}
          Stats.note_len_distribution[dur] = 1
        end
      end
           
      if MW.phr.editor_pattern then  
                
        local oct    = o.oct[i]
        local note   = o.note[i]
        local f_note = o.note[i] + 12 * o.oct[i]
        
        if note < 0 then 
          note = note + 12
          if oct >= 1 then oct = oct - 1 end
        end
      
        note,oct = oct0(clampOct (note+oct*12, Scan.Markov_oct_root,Scan.Markov_oct_range))
            
        str = f_note .. '_' .. dur
        
        if Rand.new_add_lock ~= 3 or Scan.scale_key_on then 
          table.insert(Stats.unique_notes, noteInScale(note%12))  

          Stats.pitch_duration_dst[note+1] = Stats.pitch_duration_dst[note+1] + dur     
        end
        
      elseif MW.phr.editor_chords and #o.notes >= Scan.Markov_min_notes then

        str  = ''
        o.notes[i] = unique(map(o.notes[i], function(v) return clampOct (v, Scan.Markov_oct_root,Scan.Markov_oct_range) end))

        for k,v in ipairs( o.notes[i]) do           
          --local nn = noteInScale(v%12) 
                    
          str = str .. v
     
          if k ~= # o.notes[i] then
            str = str .. '_'
          else
            str = str .. ','  .. dur
          end   
          
          if Rand.new_add_lock ~= 3 or Scan.scale_key_on then
            table.insert(Stats.unique_notes, v)
            Stats.pitch_duration_dst[v%12+1] = Stats.pitch_duration_dst[v%12+1] + dur
          end
        end        
        
    --    if Rand.new_add_lock ~= 3 or Scan.scale_key_on then
         -- each(chrd, function(v) table.insert(Stats.unique_notes, noteInScale(v%12)) end)      
         -- each(chrd, function(v) local nn = noteInScale(v%12) Stats.pitch_duration_dst[nn+1] = Stats.pitch_duration_dst[nn+1] + dur end)      
      --  end
      end
      
      if MW.phr.editor_poly then
        table.insert(Markov_LN, Line_tb[i])
      end

      table.insert(Markov_NS, str)
    end        
  end 
 
  return Markov_NS, Markov_LN  
end


function phraseScannerPoly()
  
  local ph  = MW.phr.no
  
  local NS_poly_tb, ln_tb = {},{} 

  local line = {0,0,0,0,0,0,0,0,0,0,0,0}
  local line_tb = {}
      
  for i=1,12 do 
    line_tb[i] = {}
    for k,v in pairs (MW.phr[ph].notes[i].len) do   
        
      line[i] = line[i] + v
      line_tb[i][k] = line[i]            
    end  
  end
      
  for i=1,12 do    
    local ptr = MW.phr[ph].notes[i]
    
    NS_poly_tb[i], ln_tb[i] =  phraseScannerMonoChrd(ptr, line_tb[i])   
   
    for k,v in ipairs(NS_poly_tb[i]) do  
      NS_poly_tb[i][k] =  v 
    end
  end
      
  local tb = {}
     
   for i=1,#ln_tb do 
     for j=1,#ln_tb[i] do
      local ln = ln_tb[i][j]
      local ns = NS_poly_tb[i][j]

      if tb[ln] == nil then
        tb[ln] = {}
        tb[ln] = ns
      else
        tb[ln] =  tb[ln] .. ';' ..  ns
      end          
    end
  end
      
  local tkeys = {}
  for k in pairs(tb) do table.insert(tkeys, k) end
  table.sort(tkeys)
     
  local tb_ret = {}
  for _, k in ipairs(tkeys) do table.insert(tb_ret,tb[k]) end

  return tb_ret
end


function phraseScanner()      
  
  UI.markov_first_N.items = {'Random'}
  UI.markov_first_N.value = 1   
      
  if Scan.Markov_new_add_lock == 1 or Scan.scale_key_on then
    Stats.unique_notes       = {}
    Stats.pitch_duration_dst = {0,0,0,0,0,0,0,0,0,0,0,0}
  end  
           
  if Scan.Markov_new_add_lock == 1 then 
    Stats.note_strings = {}
    Scan.Markov_process = nil

    if MW.phr.editor_poly then
      Stats.note_strings[1] = phraseScannerPoly() 
    elseif MW.phr.editor_chords then 
      Stats.note_strings[1] = phraseScannerMonoChrd(CHptr)    
    elseif MW.phr.editor_poly == false then
      Stats.note_strings[1] = phraseScannerMonoChrd(Nptr)  
    end
    
  elseif Scan.Markov_new_add_lock == 2 then      
    
    local tmp_N, tmp_NS, tmp_O, tmp_L = {},{},{},{}
    
    if MW.phr.editor_poly then
      tmp_NS = phraseScannerPoly() 
    elseif MW.phr.editor_chords then 
      tmp_NS = phraseScannerMonoChrd(CHptr)  
    else
      tmp_NS = phraseScannerMonoChrd(Nptr)  
    end

    table.insert(Stats.note_strings, tmp_NS)  
  --  sprint('Added ' .. #Stats.note_strings[#Stats.note_strings] .. ' notes' )
  end   
  
  if #Stats.note_strings > 1 then
    sprint('Loaded ' ..  #Stats.note_strings .. ' phrases')
  end
  
  if Scan.Markov_process then 
    Scan.Markov_process.params = ''
    Scan.Markov_process.ns = Stats.note_strings
  end
end
      

local function phraseScaleKeyStore(tb, scale, key, transpose, file_name)

  if tb[scale] then
    if tb[scale][key] then
      
      table.insert(tb[scale][key].file_name, file_name)
      table.insert(tb[scale][key].transpose, transpose)     
    else
      tb[scale][key] = {}
      
      tb[scale][key].file_name = {}
      tb[scale][key].transpose = {}
      
      table.insert(tb[scale][key].file_name, file_name)
      table.insert(tb[scale][key].transpose, transpose)      
    end
  else
    tb[scale] = {}
    tb[scale][key] = {}
    
    tb[scale][key].file_name = {}
    tb[scale][key].transpose = {}
    
    table.insert(tb[scale][key].file_name, file_name)
    table.insert(tb[scale][key].transpose, transpose)     
  end 
end


local function scaleKeyScan2(files,ix)

  local notes_o
  
  local fun = function (notes, pitch_duration_dst, transpose, file)
    local scale,key = findScaleKeyArg(unique(notes), pitch_duration_dst) 
      
    if scale > 1 then
      phraseScaleKeyStore(Scan.db_scale_key[ix], scale, key, transpose, file)
    end    
    
    local tb = {}
    tb.pitch_duration_dst = pitch_duration_dst
    tb.file_name = remExt2(file)
    tb.tanspose = transpose
    table.insert(Scan.db_correlation[ix], tb)    
  end
  
  for file_no,file in pairs(files) do

    sprint('File: ' .. file)
    
    local notes, offs, lens, pat_len = Load.notesPtr(file, ix)        
    
    if ix == 'chrd' then
      local ch_notes = {}
      
      for transpose = 0,11 do 
        local pitch_duration_dst = {0,0,0,0,0,0,0,0,0,0,0,0}     
        
        local line = 0
        
        for i,chord in ipairs(notes) do
          local dur = 0          
          line = line + lens[i]  
          
          if lens[i] > 0 then
            for _,note in ipairs(chord) do 
                                                           
              note = (note+transpose)%12               
              
              if (offs[i] > 0) then 
                dur = offs[i]
              elseif i < #lens and lens[i+1] > 0 then 
                dur = lens[i+1]
              else 
                dur = pat_len - line + 1
              end
              
              if dur > 0 then                        
                pitch_duration_dst[note+1] = pitch_duration_dst[note+1] +  dur  
                table.insert(ch_notes,note)
              end                        
            end
          end
        end
        
        fun(notes, pitch_duration_dst, transpose, file)      
      end      
      
      notes = ch_notes
    else       
      for transpose = 0,11 do        
       
        notes_o = {}  
              
        local pitch_duration_dst = {0,0,0,0,0,0,0,0,0,0,0,0}      
          
        for col=1,#notes do
          local notes = map(notes[col], function(v) return (v+transpose)%12 end)
          
          local line = 0
      
          for i,len in ipairs(lens[col]) do
            local dur = 0
        
            line = line + len        
        
            if len > 0 then
              if (offs[col][i] > 0) then     
                dur = offs[col][i]
              elseif i < #lens[col] and lens[col][i+1] > 0 then  
                dur = lens[col][i+1]
              else     
                dur = pat_len - line + 1
              end
            end        
         
            if dur > 0 then                      
              pitch_duration_dst[notes[i]+1] = pitch_duration_dst[notes[i]+1] +  dur  
                            
--              table.insert(notes_o,{line = line, note = notes[i], dur = dur})
            end   
          end     
        end
        
--        if ix == 'poly' then
--          table.sort(notes_o, function(a,b) return a.line < b.line end) 
--        end
--        print('notes_o')    rprint(notes_o )
        
        fun(notes, pitch_duration_dst, transpose, file)   
      end      
    end      
                            
  --  if file_no%25 == 0 then
  --    coroutine.yield()
  --  end
  end     
end


function scaleKeyScan()
  Scan.db_correlation = {}
  Scan.db_correlation.mono = {}
  Scan.db_correlation.poly = {}
  Scan.db_correlation.chrd = {}
  
  Scan.db_scale_key = {}
  Scan.db_scale_key.mono = {}
  Scan.db_scale_key.poly = {}
  Scan.db_scale_key.chrd = {}
    
  scaleKeyScan2(getFiles('.seq' ),'mono')    
  scaleKeyScan2(getFiles('.seqs'),'poly')  
  scaleKeyScan2(getFiles('.chrd'),'chrd')  

  sprint('Done')
end


function phraseCorrelation() 
 
  if #Scan.db_correlation == 0 then return {} end

  local pitch_duration_dst = copy(Stats.pitch_duration_dst) 
 
  local len = #UI.lib_pu.items
  
  local best_c = -9999
  local best_fn
  local best_tb = {}
  
  local correlating_files = {}
  
  local ix      
  if MW.phr.editor_poly and MW.phr.editor_chords == false then
    ix =  'poly'
  elseif MW.phr.editor_chords then
    ix = 'chrd'
  else
    ix = 'mono'
  end

  UI.load_scale_key.value = false
    
  local filename_src = UI.lib_pu.items[UI.lib_pu.value]
--  local tmp_ms, tmp_rs = Scan.Markov_new_add_lock, Rand.new_add_lock              
--  Scan.Markov_new_add_lock, Rand.new_add_lock = 1,1
                  
  for k,v in ipairs (Scan.db_correlation[ix]) do 
    sprint('Left ' .. #Scan.db_correlation[ix] - k)
       
    local c = pearson2tbs(pitch_duration_dst, v.pitch_duration_dst) 
    if filename_src ~= v.file_name and c > best_c then 
      best_c = c  
      best_fn = v.file_name
      table.insert(best_tb, best_fn)
      table.insert(correlating_files,v.tanspose)
    end                
  end 
        
--  Scan.Markov_new_add_lock, Rand.new_add_lock = tmp_ms, tmp_rs
  
  reverseTb(correlating_files)
  reverseTb(best_tb)
  return best_tb
end


function getChordNotesDistrib() 
  
  Stats.unique_notes       = {}
  Stats.pitch_duration_dst = {0,0,0,0,0,0,0,0,0,0,0,0}
    
  local line = 0
 
  for i=1,#CHptr.len do  
    local len = CHptr.len[i]
    local dur = 0
    local str
    
    line = line + len      
 
    if len > 0 then
      if (CHptr.off[i] > 0) then 
        dur = CHptr.off[i]
      elseif i < #CHptr.len and CHptr.len[i+1] > 0 then  
        dur = CHptr.len[i+1]
      else  
        dur = CHptr.pat_len - line + 1
      end     
    end

    if dur > 0 then    
      for k,v in ipairs(CHptr.notes[i]) do    
        table.insert(Stats.unique_notes, v)
        Stats.pitch_duration_dst[v%12+1] = Stats.pitch_duration_dst[v%12+1] + dur
      end   
    end        
  end 
end

   