
---------------------------------------------- SECOND ROW  -----  ui scales

function ui_scales ()
  
  local vb = renoise.ViewBuilder()

  local popup_scales = vb:popup {
    width = 150,
    value = 1,
    items = Scales.names,    
  }    
    
  local popup_keys = vb:popup {
    width = 45,
    value = 1,
    items = Scales.keys,    
  }      
 
 local scale_sw = vb:switch {
     width = 60,
     value = Scales.on,
     tooltip = "ON - grid displays notes that only belong to selected scale and key. OFF - scale notes are denoted with dots '.'",
     items = {"ON", "OFF"},
     notifier = function(val)    
       Scales.on = val
    end  
  }    
    
  local scales_row = vb:row {
    spacing = 5,
  
    vb:text {
      width = 30,
      text = "Scale "
    },
    popup_scales,
    scale_sw,
  }
 
  
  local update_bt = vb:button {
    tooltip = "Update notes and editor with selected scale and key",
    text = " UD ",
    notifier =  function ()
      if MW.edit_RN_phrase then
        local inst_no = renoise.song().selected_instrument_index
        
        renoise.song().instruments[inst_no].trigger_options.scale_key = popup_keys.value
        
        local scales_rn = renoise.song().instruments[inst_no].trigger_options.available_scale_modes          
        renoise.song().instruments[inst_no].trigger_options.scale_mode = scales_rn[popup_scales.value]        
      else    
        if Scales.on == 1 then  
          local ptr = MW.dup[MW.phr.no].notes
                    
          if MW.phr.editor_poly then  
                            
            for i=1,12 do                   
              Nptr.note = map(ptr[i].note, function(v) return noteInScale(v) end)                                
              fill(i)   
            end                                        
          else          
            Nptr.note = map(ptr[MW.phr.col].note, function(v) return noteInScale(v) end)  
            fill(MW.phr.col)
          end              
        end
         
        updateSlots()
      end      
    end 
  }  
   
     
  local scale_sel = vb:button {
    tooltip = "Select first scale with most matching notes.",
    text = "Sel",
    notifier =  function ()

      if #Scales.best_scale > 0 and #Scales.best_key > 0 then
      
        UI.scales_pu.value = Scales.best_scale[1]
        UI.keys_pu.value = Scales.best_key[1]
      end       
    end 
   }  
  
  local function loadAll() 
    local len = #UI.lib_pu.items
    
    if len > 0 then
      UI.suspend_update = true
      local tmp_ms, tmp_rs = Scan.Markov_new_add_lock, Rand.new_add_lock              
            
      if len > 1 then                    
        for i=1,len do
          sprint('Files left ' .. len - i)
          if i == 1 then 
            Scan.Markov_new_add_lock, Rand.new_add_lock = 1,1
          else
            Scan.Markov_new_add_lock, Rand.new_add_lock = 2,2   
          end             
             
          local  filename = MW.path .. UI.lib_pu.items[i] .. Lib.file_ext
            
          if MW.phr.editor_poly and MW.phr.editor_chords == false then
            readSeq(filename)
          elseif MW.phr.editor_chords then
            readSeqChord(filename) 
          else
            readSeqM(filename) 
          end
          coroutine.yield()  
        end
      end
        
      Scan.Markov_new_add_lock, Rand.new_add_lock = tmp_ms, tmp_rs        
      UI.suspend_update = false
      
      sprint('Done')
    end
  end  
  
  
  local load_all_bt = vb:button {
    tooltip = "Load all phrases to note generator / mutator",
    text = "Load All",
    width = 50,
    visible = false,
    notifier =  function ()   
      local process = ProcessSlicer(loadAll, function() end)
      process:start()     
    end 
  }     
  
  
  local scan_last_val = 0 
  
  local scan_sw
  scan_sw = vb:switch {
     width = 100,
     value = 1,
     tooltip = "Limit phrase selection based on: COR - correlation, SK - same scale and key, NA - all files",
     items = {'NA', 'COR', 'SK'},
     notifier = function(val)    
 
       if val == 2 then   
       
        local fns = phraseCorrelation()
        
        if #fns>0 then
          UI.lib_pu.items = fns
          local ix 
          if MW.phr.editor_poly == false and MW.phr.editor_chords == false then
            ix = 'mono'
          elseif MW.phr.editor_poly then
            ix = 'poly'  
          elseif MW.phr.editor_chords then
            ix='chrd'
          end 
          Scan.correlation_on = true 
        else
          sprint('Not found. Make sure phrases were scanned (Setup, Scan)')
        end
        
       elseif scan_last_val == 2 then
         UI.lib_pu.items = remExt(os.filenames(MW.path, {"*" .. Lib.file_ext})) 
         Scan.correlation_on = false    
       end
      
      
      if val == 3 then       
        local scale = UI.scales_pu.value 
        local key = UI.keys_pu.value  
        
        local ix 
        if MW.phr.editor_poly == false and MW.phr.editor_chords == false then
          ix = 'mono'
        elseif MW.phr.editor_poly then
          ix = 'poly'  
        elseif MW.phr.editor_chords then
          ix='chrd'
        end 
        
      ---------------- scale key  
      if val == 3 and popup_scales.value == 1 then 
        sprint('Select scale.')            
                       
      elseif val == 3 and scale > 1 and Scan.db_scale_key[ix][scale] then 
          if Scan.db_scale_key[ix][scale][key] then
            UI.lib_pu.items = remExt(Scan.db_scale_key[ix][scale][key].file_name)

            Scan.scale_key_on = true 
            load_all_bt.visible = true
            UI.load_scale_key.value = false
            return
          end
        end  
        
        sprint('Not found. Make sure phrases were scanned (Setup, Scan)')
        
        scan_sw.value = 1
        Scan.scale_key_on = false
        load_all_bt.visible = false    
        UI.load_scale_key.value = false         
      elseif val == 3 or scan_last_val == 3 then 
        UI.lib_pu.items = remExt(os.filenames(MW.path, {"*" .. Lib.file_ext})) 
        Scan.scale_key_on = false
        load_all_bt.visible = false    
        UI.load_scale_key.value = true 
      end       
      
      scan_last_val = val
    end  
  }    
    
  local keys_row = vb:row {
    spacing = 3,
    vb:text {
      width = 25,
      text = "key"
    },
    popup_keys,
    
    update_bt,
    scale_sel,    
    scan_sw,
    load_all_bt,
  }    
      
  local scales_col = vb:column {
    style = 'group',
    spacing = 3,
    margin = 5,
      scales_row,
      keys_row,  
  }
  
  UI.scales_pu = popup_scales
  UI.keys_pu = popup_keys
  UI.scale_sw = scale_sw
  
  return scales_col      
end 

