local setup_dialog

function SetupPrompt()

  local vb = renoise.ViewBuilder()
  
  if setup_dialog  and setup_dialog.visible then
     setup_dialog:show()  
     return 
  end

  local tx = vb:text {
    text = "Folder:  " .. MW.path
  }  
  
  
  local but_ok = vb:button {
    text = "    OK    ",
    notifier =  function ()
      if Lib.temp_path ~= nil and io.exists(Lib.temp_path) then
        preferences.path.value = Lib.temp_path
        MW.path = Lib.temp_path          
        setup_dialog:close()
      else
        sprint('No new valid folder selected')
        setup_dialog:close()
      end 
    end 
  }
       
  local but_cancel = vb:button {
    text = "  Cancel  ",
    notifier =  function ()
      setup_dialog:close() 
    end 
  }       

  local but_new_folder = vb:button {
    text = " Select New Folder ",
    notifier =  function ()
      Lib.temp_path = renoise.app():prompt_for_path("Select Library Folder")
      tx.text =  "Folder:  " .. Lib.temp_path
    end 
  }
       
  local scale_key_scan = vb:button {
    text = " Scale&Key Scan ",
    notifier =  function ()   
      
   --   local process = ProcessSlicer(scaleKeyScan, function() end)
    --  process:start()
      scaleKeyScan()
 
   
      local fn = MW.path .. 'scan.lua'        
      if fileExists(fn) then
        os.remove (fn)   
      end
        
      local file = io.open(fn, "wb")   
      sprint('Dump and save scan.lua')
      file:write('Scan.db_scale_key = ' .. serialize(Scan.db_scale_key))
      file:close()    
        
        
      fn = MW.path .. 'scan2.lua'        
      if fileExists(fn) then
        os.remove (fn)   
      end
        
      file = io.open(fn, "wb") 
      sprint('Dump and save scan2.lua')
      file:write('Scan.db_correlation = ' .. serialize(Scan.db_correlation))
      file:close()         
    end 
  }
  
           
  local combine_but = vb:button {
    tooltip = 'Combine notes from first column of unmuted tracks to selected track',
    text = ' Comb ',
    notifier =  function ()
      combine(MW.phr.editor_poly)
    end 
  }

  local combineR_but = vb:button {
    tooltip = 'Combine notes from first column of unmuted tracks to selected track',
    text = ' CombR ',
    notifier =  function ()
      combine(MW.phr.editor_poly, "rep")
    end 
  }
  
  local combine_poly_but = vb:button {
    text = ' CombPoly ',
    tooltip = 'Combine notes from unmuted tracks to selscted track',
    notifier =  function ()
      combinePoly()
    end 
  }
  
  local remap_instruments_but = vb:button {
    tooltip = 'Remap all instruments in a pattern or sequence to selected instrument',
    text = ' Remap Instruments',
    notifier =  function ()
      remapInst()
    end 
  }
        
  local newLen_vb = vb:valuebox {
    min = 1,
    max = 128,
    width = 50,
    value = 64,
  }

   local newLen_but = vb:button {
    text = " Block Resize ",
    notifier =  function ()
      resizeBlock (newLen_vb.value)
    end 
  }   

  local mutate_sw = vb:switch {
    width = 100,
    value = Rand.mutate_markov,
    tooltip = "mutate using simple randomization or Markov chain",
    items = {"Random", "Markov"},
    notifier = function(val)    
      Rand.mutate_markov = val
   end  
  }
       
  local max_len_vb = vb:valuebox {
    min = 1,
    max = 512,
    width = 60,
    value = MW.grab_max_len,
    tooltip = "Grab maximum note length",
    notifier = function(val)     
      MW.grab_max_len = val
    end
  }      

  local min_len_vb = vb:valuebox {
    min = 0,
    max = 99,
    width = 60,
    value = MW.grab_min_len,
    tooltip = "Grab minimum note length",
    notifier = function(val)     
      MW.grab_min_len = val
    end
  }  

  local markov_notes_len_txf =  vb:textfield {
    width = 120,
    align = "left",
    tooltip = 'Coma separated note lengths to be used with Markov chains (to reduce number of combinations). If the field is empty, all lengths are used.',
    text = Stats.note_strings_length,
    notifier = function(text)       
      Stats.note_strings_length = text
    end
  }
  
  local markov_clamp_root_vb = vb:valuebox {
      min = 0,
      max = 64,
      width = 45,
      value = Scan.Markov_oct_root,
      tooltip = "Root osctave",
      notifier = function(val)     
        Scan.Markov_oct_root = val
      end
    }  
    
  local markov_clamp_oct_vb = vb:valuebox {
      min = 0,
      max = 64,
      width = 45,
      value = Scan.Markov_oct_range,
      tooltip = "Clamp above and belove specified octave, 0 dont't clamp",
      notifier = function(val)     
        Scan.Markov_oct_range = val
      end
    } 
    
  local markov_min_notes_vb = vb:valuebox {
      min = 1,
      max = 3,
      width = 45,
      value = Scan.Markov_min_notes,
      tooltip = "Minmum number of notes",
      notifier = function(val)     
        Scan.Markov_min_notes = val
      end
    }     
     
  local profiles_pu = vb:popup {
    width = 120,
    value = Scales.profiles.no,
    items = Scales.profiles.pu,
    tooltip = "Scale and key Scales.profiles",
    notifier = function(new_index)
      Scales.profiles.no = new_index
    end
  }
        
  local dialog_content = vb:column { 
    margin = 10,
    spacing = 6,
    
    scale_key_scan,
    
    vb:row{
      combine_but,
      combineR_but,      
    },
    combine_poly_but,
    remap_instruments_but,
    
    vb:row{
      newLen_but,
      newLen_vb,
    },   
        
    vb:text {
      width = 60,
      text = ' Lib mutate '
      },
    mutate_sw,

    vb:row {
      vb:text {
        width = 60,
        text = ' Grab min note L '
        },
        min_len_vb,
    },
    
    vb:row {
      vb:text {
        width = 60,
        text = ' Grab max note L '
        },
        max_len_vb,
    },

    vb:text {
      width = 60,
      text = ' Markov Note '
    },

    vb:text {
      width = 80,
      text = 'Markov Note Lengths'
    },       
    markov_notes_len_txf,          
    
    vb:text {
      width = 80,
      text = 'Markov Note Clamp'
    },
    
    vb:row{
      vb:text {
        width = 50,
        text = 'Root Oct '
      },
      markov_clamp_root_vb,
    },
    
    vb:row{
      vb:text {
        width = 50,
        text = 'Range '
      },
      markov_clamp_oct_vb,      
    },
    
    markov_min_notes_vb,
    
     vb:text {
      width = 80,
      text = 'Scale and key profile'
    }, 
    profiles_pu,
    
    tx,
    but_new_folder,       
    vb:row {
      but_ok,
      but_cancel,
    },
  }    

  setup_dialog = renoise.app():show_custom_dialog('Setup, Utilities', dialog_content, function(dialog,key) return key end)      
end

